package cfgfile;

use strict;

use FileHandle;

my %vars = ();

sub new
{
    my ($this, @files) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $self->{"files"} = [];
    $self->{"path"} = "$main::root/adm:$main::root/etc:$main::root:.";
    $self->{"watchfiles"} = [];
    $self->{"mtimes"} = [];
    $self->{"fh"} = new FileHandle();
    $self->{"read_files"} = 0;
    my $file;
    foreach $file (@files) {
	$self->add_file( $file );
    }

    unless( %vars ) {
	$vars{"HOST"} = $main::hostname;
    }
    return $self;
}


sub find_file {
    my( $self, $file ) = @_;
    my $tf = "";

    while( $file =~ /\$(\$|[A-Z]+)/ ) {
	$tf .= $`;
	$file = $';
	if( $1 eq "\$\$" ) {
	    $tf .= "\$";
	}
	else {
	    $tf .= $vars{$1};
	}
    }
    $file = $tf.$file;
    if( $file =~ /^\// ) {
	return( $file );
    }
    my $dir;
    foreach $dir (split( ":", $self->{"path"})) {
	if( -f $dir."/".$file ) {
	    return( $dir."/".$file );
	}
    }
    return undef;
}


sub add_watch {
    my( $self, $file ) = @_;

    return if( grep( $_ eq $file, @{$self->{"watchfiles"}} ) );
    push( @{$self->{"watchfiles"}}, $file );
    push( @{$self->{"mtimes"}}, ((stat($file))[9]) );
}


sub add_file {
    my( $self, $file, $position ) = @_;

    if( ! defined $position ) {
	$position = $#{$self->{"files"}} + 1;
    }
    $file = $self->find_file( $file );
    if( $file ) {
	return(1) if( grep( $_ eq $file, @{$self->{"files"}} ) );
	splice( @{$self->{"files"}}, $position, 0, $file );
	$self->add_watch( $file );
	return 1;
    }
    return undef;
}


sub has_changed {
    my( $self ) = @_;
    my $i;

    for( $i=0; $i<=$#{$self->{"watchfiles"}}; $i++ ) {
	my $mtime = ((stat( $self->{"watchfiles"}->[$i] ))[9]);
	if( $mtime != $self->{"mtimes"}->[$i] ) {
	    return( $self->{"watchfiles"}->[$i] );
	}
    }
    return undef;
}


sub rewind {
    my( $self ) = @_;

    $self->{"fileno"} = 0;
    $self->{"includes"} = [];
    $self->{"watchfiles"} = [];
    $self->{"mtimes"} = [];
    $self->_open_next();
}


sub _open_next {
    my( $self ) = @_;

    if( $#{$self->{"includes"}} >= 0 ) {
	$self->{"fileopen"} = 1;
	$self->{"fh"} = shift @{$self->{"includes"}};
	return 1;
    }
    while( $self->{"fileno"} <= $#{$self->{"files"}} ) {
	$self->add_watch( $self->{"files"}->[$self->{"fileno"}] );
	$self->{"fileopen"} = 
	  open( $self->{"fh"}, "<".$self->{"files"}->[$self->{"fileno"}] );
	$self->{"fileno"}++;
	return(1) if( $self->{"fileopen"} );
    }
    return undef;
}


sub nextline {
    my( $self ) = @_;

    return(undef) if( ! $self->{"fileopen"} );
    my $fh = $self->{"fh"};
    my $line = <$fh>;
    unless( $line ) {
	close( $self->{"fh"} );
	$self->{"fileopen"} = 0;
	$self->_open_next();
	return( $self->nextline() );
    }
    if( $line =~ /^[\s\t]*#/ ) {
	return( $self->nextline() );
    }
    if( $line =~ /\\[\r\n]*$/ ) {
	$line = $`;
	$line .= $self->nextline();
    }
    if( $line =~ /^include[\s\t]+([^\s\t]+)[\r\n]*$/ ) {
	$self->_include( $1 );
	return( $self->nextline() );
    }
    return $line;
}


sub _include {
    my( $self, $file ) = @_;

    $file = $self->find_file( $file );
    return( undef ) unless( $file );
    $self->add_watch( $file );
    my $fh = new FileHandle();
    open( $fh, "<$file" ) || return(undef);
    push( @{$self->{"includes"}}, $self->{"fh"} );
    $self->{"fh"} = $fh;
}

1;

