#    Big Sister network monitor
#    Copyright (C) 1998  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

use Platforms;
use strict;
use Getopt::Long;


sub proginit {
    $main::domain_encode = ".";
    %main::status_codes = (
	"red" => 0,
	"yellow" => 1,
	"purple" => 2,
	"green" => 3,
	"clear" => 4,
	"blue" => 5,
	"white" => 6
    );

    %main::status_texts = (
	0 => "red",
	1 => "yellow",
	2 => "purple",
	3 => "green",
	4 => "clear",
	5 => "blue",
	6 => "white"
    );

    $main::log_facility = (Platforms::iswin32)?"Srv":"daemon";
    $main::progdir = $0;
    $main::prog = $0;
    $main::prog =~ s#.*[/\\]##;
    $main::progdir =~ s#/[^\/\\]*$##;
    my $bs_instdir="/usr/lib/bs";

    if( Platforms::isunix() ) {
	eval {
	    require Sys::Syslog;
	    $main::syslog = 1;
	};
    }
    if( Platforms::iswin32() ) {
	eval {
	    require Win32::EventLog;
	    $main::eventlog = 1;
	};
    }
     
    $main::root = Platform::override_root();
    $main::root = $bs_instdir unless( $main::root );
    $Getopt::Long::ignorecase = 0;  
    GetOptions( "D:i", "b:s", "L:s", @main::options ) || &usage;
    $main::dl = $main::opt_D;
    if( $main::opt_b ) {
	$main::root = $main::opt_b;
    }
    load_resources();
    if( $main::resources{"log_facility"} ) {
	$main::log_facility = $main::resources{"log_facility"};
    }
    if( $main::opt_L ) {
	$main::log_facility = $main::opt_L;
    }

    if( $main::resources{"path"} ) {
	$main::resources{"path"} =~ /^(.*)$/;
	if( Platforms::iswin32 ) {
	    $ENV{"PATH"} = $1.";".($ENV{"PATH"});
	}
	else {
	    $ENV{"PATH"} = $1;
	}
    }

    unshift( @INC, $main::root );
    if( $main::log_facility && ! $main::syslog && ! $main::eventlog ) {
	print STDERR "$main::prog: WARNING: syslog not available - falling back to logger\n" if( $main::dl );
    }

    if( $main::syslog ) {
	eval {
	    $main::syslog = 0;
	    Sys::Syslog::setlogsock( 'unix' );
	    Sys::Syslog::openlog( $main::prog, "", $main::log_facility );
	    $main::syslog = 1;
	};
    }
    $main::eventlog = Win32::EventLog->new( $main::log_facility ) if( $main::eventlog );
    &log( "debug", "starting up" );
    $main::hostname = Platform::hostname();
}



sub usage {
    print "Usage: ".($main::prog)." ".($main::Usage)."\n";
    exit 1;
}


sub background {
    &log( "info", "entering daemon mode" );
    if( Platforms::isunix() ) {
	chdir "/";
	close STDIN; close STDOUT; close STDERR;
	if (open(DEVTTY, "/dev/tty")) {
	    ioctl(DEVTTY,0x20007471,0);         # XXX s/b &TIOCNOTTY
	    close DEVTTY;
	}
	open(STDIN,"</dev/null");
	open(STDOUT,">/dev/null");
	open(STDERR,">&STDOUT");
	fork && exit;
	$SIG{"HUP"} = $SIG{"ALRM"} = $SIG{"INT"} = $SIG{"PIPE"} = "IGNORE";
	setpriority( "PRIO_PROCESS", 0, 10 );
    }
    my $pidfile = $main::prog;
    $pidfile =~ s/\.exe$//i;
    open( PID, ">$main::root/adm/$pidfile.pid" );
    print PID "$$\n";
    close PID;
}


sub load_resources {
    my $file;
    %main::resources = ();
    foreach $file( "/etc/resources", "/adm/resources" ) {
	if( -f $main::root.$file ) {
	    open( RES, "<$main::root$file" );
	    while( <RES> ) {
		chomp;
		if( /^($main::prog|\*)\.([a-zA-Z0-9]+)[\s\t]*=[\s\t]*(.*)/ ) {
		    $main::resources{$2} = $3;
		}
	    }
	    close RES;
	}
    }
}


sub log {
    my( $severity, @messages ) = @_;
    my( $message, $level );

    if( $severity eq "warn" ) { $severity = "warning"; }
    foreach $message (@messages) {
	$level = 3;
	($severity =~ /(alert|err|crit)/) && ($level = 1);
	($severity =~ /(warning|notice)/) && ($level = 2);
	($severity =~ /(info)/) && ($level = 3);
	if( ($main::dl && ($severity =~ /(alert|err|crit)/))
	    || ( ($main::dl>1) && ($severity =~ /(warning|notice)/) )
	    || ( ($main::dl>2) && ($severity =~ /(info)/) )
	    || ($main::dl>3) ) {
	    print STDERR (scalar localtime).": ".$main::prog.": ".$message."\n";
	}
	next unless( $main::log_facility );
	if( $main::syslog ) {
	    eval {
		$main::syslog = 0;
		Sys::Syslog::syslog( $severity, "\%s", $message );
		$main::syslog = 1;
	    };

	}
	elsif( $main::eventlog ) {
	    my %event = ('EventID' => 100, 
		'EventType' => $level, 
		'Strings' => $main::prog.": ".$message, 
		"Source" => "Perl" );

	    $main::eventlog->Report( \%event );
	}
	unless( $main::eventlog || $main::syslog ) {
	    $severity =~ s/\|.*//;
	    system( "logger", "-p", $main::log_facility.".".$severity, "-i", $main::prog.":", $message );
	}
    }
}

	    
sub parse {
    my( $str ) = @_;
    my @res = ();
    my $buff = "";

    while( $str =~ /^([^\s\t\"\']*)([\s\t]+|[\"\'])/ ) {
	my( $arg, $sep ) = ($1,$2,$');
	$str = $';

	if( $sep =~ /[\s\t]/ ) {
	    push( @res, $buff.$arg );
	    $buff = "";
	}
	else {
	    $buff .= $arg;
	    if( $str =~ /^(.*?)\Q$sep\E/ ) {
		my $rest = $1;
		$str = $';
		if( $sep eq '"' ) {
		    my( $prefix );
		    while( $rest =~ /\\(.)/ ) {
			my $next = $1;
			$rest = $';
			$prefix .= $`;
			if( $next eq "n" ) {
			    $next = "\n";
			}
			elsif( $next eq "r" ) {
			    $next = "\r";
			}
			$prefix .= $next;
		    }
		    $rest = $prefix.$rest;
		}
		print "********$rest\n";
		$buff .= $rest;
	    }
	    else {
		$buff .= $str;
		$str = "";
	    }
	    if( $str =~ /^[\s\t]+/ ) {
		$str = $';
		push( @res, $buff );
		$buff = "";
	    }
	}
    }
    $str = $buff.$str;
    push( @res, $str ) if( $str );
    return @res;
}


sub separate_service {
    my( $item ) = @_;

    if( $item =~ /^(.*)\.(.*?)$/ ) {
	return( $1, $2 );
    }
    return( $item );
}
1;
