package snmp;

use Exporter;
use Socket;

@EXPORT = qw (myEntOID myOID mySNMPtraps);

my $myEntOID = "1.3.6.1.4.1.5281";

my %myOID = (
    "enterprise", "1.3.6.1.4.1",
    "graeff", $myEntOID,
    "bshost", $myEntOID.".1.1.1",
    "bsservice", $myEntOID.".1.1.2",
    "bsstatus", $myEntOID.".1.1.3",
    "bscomment", $myEntOID.".1.1.4",
    "bsseverity", $myEntOID.".1.1.5",
    "bsackmode", $myEntOID.".1.1.6",
    "bsackuser", $myEntOID.".1.1.7"
);

my %mySNMPtraps = (
    "coldstart", [ 0, 0 ],
    "bsstatuschange", [6, 1],
    "bsserviceup", [6, 2],
    "bsservicedown", [6, 3],
    "bsalarmack", [6, 4],
    "bsremind", [6, 5]
);

my $have_snmp = 0;

eval {
    require BER;
    require SNMP_Session;
    require SNMP_util;
    $have_snmp = 1;
};


sub trap {
    my( $host, $community, $agent, $type, $args ) = @_;

    print STDERR "snmp::trap to $host ($community) from $agent of type $type\n" if( $main::dl>4 );
    return unless $have_snmp;
    if( (! $community) && ($host =~ /^(.*)\@(.*)$/) ) {
	$community = $1;
	$host = $2;
    }
    ($community = "public") unless( $community );
    my $itype;
    unless( $type =~ /^\d+$/ ) {
	$itype = $mySNMPtraps{$type};
	unless( $itype ) {
	    main::log( "warning", "internal error: invalid SNMP trap" );
	}
    }
    else {
       $itype = {6,$type};
    }

    my $arg;
    my @args;
    foreach $arg (sort { $myOID{$a} cmp $myOID{$b} } keys %$args) {
	my $oid = $myOID{$arg};
	next unless( $oid );
	push( @args, $oid, @{$args->{$arg}} );
    }

    $agent =~ s/.*://;
    unless( $agent && inet_aton($agent) ) {
	$agent = Platform::hostname();
    }
    my $result = SNMP_util::snmptrap( 
	$community."\@".$host, 
	$myOID{"graeff"},
	$agent,
	$itype->[0],
	$itype->[1],
	@args );
    unless( $result ) {
	main::log( "warning", "SNMP trap $type to $host failed" );
    }
}






1;
