package Monitor::bb;

require 5.002;
require Exporter;

use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Socket qw( SOCK_STREAM PF_INET inet_aton sockaddr_in inet_ntoa INADDR_ANY);
use FileHandle;
use Carp;

@ISA = qw(Exporter Monitor::Monitor);
$Monitor::bb::vardump = 0;


sub new
{
    my ($this, $host, $timeout, $port) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    print "Monitor::bb new()\n" if( $main::dl>1 );
    bless($self, $class);

    ($timeout = 8) if( ! $timeout );
    $self->{"timeout"} = $timeout;

    ($port = "bb") if( ! $port );
    unless( $port =~ /^\d+$/ ) { $port = (getservbyname($port, 'tcp'))[2]; }
    ($port = 1984) if( ! $port );

    $self->{"port"} = $port;
    $self->{"host"} = $host;
    $self->{"proto_num"} = (getprotobyname('tcp'))[2];
    $self->{"fh"} = FileHandle->new();
    $self->{"FQDN_sep"} = $main::domain_encode;
    $self->{"FQDN"} = 1;
    $self->{"bind_ip"} = sockaddr_in(0, INADDR_ANY);
    $self->{"retries"} = [];
    $self->{"perf_resolveinterval"} = 60;

    return($self);
}


sub perfdata {
    my( $self, $file ) = @_;
    my @perf = ();

    unless( $file =~ /^\// ) {
	$file = $main::root."/".$file;
    }
    open( _IN, "<$file" );
    while( <_IN> ) {
	chomp;
	print STDERR "bb:perfdata: adding $_\n" if( $main::dl>7 );
	push( @perf, $_ );
    }
    $self->{"perfraw"} = \@perf;
    $self->{"perf"} = [];
    $self->{"perf_resolve"} = 0;
    close _IN;
}


sub options {
    my( $self, @options ) = @_;
    my $opt;

    $self->{"options_set"} = 1;
    foreach $opt ( @options ) {
	if( grep( $opt eq $_, "perf", "status", "group" ) ) {
	    $self->{"option-$opt"} = 1;
	}
    }
}



sub perfdata_resolve {
    my( $self ) = @_;
    my( $var, $pattern, $dump );

    return if( time - $self->{"perf_resolve"} < $self->{"perf_resolveinterval"} );
    open( DUMP, ">$main::root/var/uxmon_variables" );
    $Monitor::bb::vardump = $dump;
    my @vars = ();
    my @names = ();
    my $host;
    foreach $host (keys %uxmon::monitor_variables) {
	push( @names, map( $host.":".$_, keys %{$uxmon::monitor_variables{$host}} ) );
	my( $var, $val );
	while( ($var,$val) = each %{$uxmon::monitor_variables{$host}} ) {
	    print DUMP "$host:$var = $val\n";
	}
    }
    close(DUMP);
    foreach $pattern ( @{$self->{"perfraw"}} ) {
	eval {
	    push( @vars, grep( /^$pattern$/, @names) );
	}
    }
    $self->{"perf"} = \@vars;
    print STDERR "perfdata_resolve: patterns resolved to ".($#vars+1)." variables\n";
    $self->{"perf_resolve"} = time;
    $self->{"perf_resolveinterval"} += 480;
    if( $self->{"perf_resolveinterval"} > 4*3600 ) {
	$self->{"perf_resolveinterval"} = 4*3600;
    }
}



sub fqdn {
    my( $self, $fqdn ) = @_;

    $self->{"FQDN"} = $fqdn;
}

sub bind_ip {
    my( $self, $ip ) = @_;

    $self->{"bind_ip"} = sockaddr_in(0, inet_aton($ip));
}

sub bb_compat {
    my( $self, $compat ) = @_;

    $self->{"compat"} = $compat;
    $self->{"FQDN_sep"} = ($compat?",":$main::domain_encode);
}


sub set_time {
    my( $self, $time ) = @_;

    $self->{"override_time"} = $time;
}


sub run {
    my( $self, @args ) = @_;
    $self->report( @args );
}


sub report
{
    my ($self, $status, $comment, $join) = @_;
    my( $date, $item, $item_status, $ip, $shitem, $add, $i );

    my @retries = ();
    $ip = inet_aton( $self->{"host"} );
    unless( $ip ) {
	print "to bb: error, host unknown: ".$self->{"host"}."\n" if( $main::dl>1 );
	return;
    }
    $self->_start_report( $ip ) unless( $self->{"compat"} );
    $date = time unless( $date = $self->{"override_time"} );
    if( (! $self->{"compat"}) && ((! $self->{"options_set"}) || $self->{"option-group"}) ) {
	while( ($item, $item_status) = each( %$join ) ) {
	    next unless( @$item_status );
	    $shitem = substr( $item, 1 );
	    $add = substr( $item, 0, 1 );
	    ($add eq "+") && ($add = "join");
	    ($add eq "-") && ($add = "leave");
	    ($add eq "D") && ($add = "displayname");
	    $self->report_hw( "$add $shitem ".join(" ",@$item_status) );
	}
    }

    if( defined( $self->{"perf"} ) && ! $self->{"compat"} &&
	((! $self->{"options_set"}) || $self->{"option-perf"})) {
	my $var;

	$self->perfdata_resolve();
	foreach $var ( @{$self->{"perf"}} ) {
	    $var =~ /^(.*?):(.*)$/;
	    my $msg = "perf ".time." $var ".($uxmon::monitor_variables{$1}->{$2});
	    $msg =~ s/\n/|>/g;
	    $self->report_hw( $msg ) || push( @retries, $msg );
	}
    }
    if( (! $self->{"options_set"}) || $self->{"option-status"} ) {
	while( ($item,$item_status) = each( %$status ) ) {
	    print "to bb: $item: $item_status (".$$comment{$item}.")\n" if( $main::dl>3 );
	    $self->_start_report( $ip ) if( $self->{"compat"} );
	    $self->report_one( $ip, $item, $item_status, $$comment{$item}, $date ); 
	    $self->_end_report( $ip ) if( $self->{"compat"} );
	}
    }
    my $retryref = $self->{"retries"};
    for( $i=$#$retryref+1; $i>=0; $i-- ) {
	if( $self->report_hw( $retryref->[$i] )) {
	    splice( @$retryref, $i, 1 );
	}
    }
    push( @$retryref, @retries );
	
    $self->_end_report unless( $self->{"compat"} );
}



sub sysreport {
    my( $self, $msg ) = @_;

    unless( $self->{"connected"} ) {
	return if( $self->{"connectfailure"} );
	my $ip = inet_aton( $self->{"host"} );
	unless( $ip ) {
	    print "to bb: error, host unknown: ".$self->{"host"}."\n" if( $main::dl>1 );
	    return;
	}
	$self->_start_report( $ip );
	$self->{"connectfailure"} = ! $self->{"connected"};
    }
    return unless( $self->{"connected"} );
    $self->report_hw( $msg );
    if( $self->{"compat"} ) {
	$self->_end_report();
    }
}



sub _start_report
{
    my ($self, $ip) = @_;
    my ($timeout, $saddr );
    my( $port, $res );

    $timeout = $self->{"timeout"};
    $port = $self->{"port"};

    socket($self->{"fh"}, &PF_INET(), &SOCK_STREAM(), $self->{"proto_num"}) || return( 0 );
    my ($tp, $ti) = sockaddr_in($self->{"bind_ip"});
    print "Binding to ", $ti, "\n" if ( $main::dl );
    bind($self->{"fh"}, $self->{"bind_ip"}) || print "Could not bind\n";
    $saddr = sockaddr_in($port, $ip);

    $res = $self->{"connected"} = 0;
    eval {
	$SIG{'ALRM'} = sub { die "Monitor::bb:_start_report timed out" };
	Platform::bsalarm($timeout); 
	    
	if( Platform::bsconnect($self->{"fh"}, $saddr) ) {
	    $self->{"connected"} = $res = 1;
	}
	Platform::bsalarm(0);
    };
    Platform::bsalarm(0);
    close $self->{"fh"} unless( $res );
    return $res;
}



sub _end_report {
    my( $self ) = @_;

    select( undef, undef, undef, 0.5 );
    close( $self->{"fh"} );
    select( undef, undef, undef, 0.1 );
    $self->{"connected"} = 0;
    $self->{"connectfailure"} = 0;
}



sub report_one
{
    my ($self, $ip, $item, $status, $comment, $date) = @_;

    if( $self->{"FQDN"} ) {
        $item =~ s/[\.$main::domain_encode](.*)\./\./;
    }
    else {
        my @tok = split( /[\.$main::domain_encode]/, $item );
        my $check = pop @tok;
        $item = join( $self->{"FQDN_sep"}, @tok ).".".$check;
    }

    if( $self->{"compat"} ) {
	$comment =~ s/\|\>/\n/g;
    }
    else {
	$comment =~ s/\n/\|\>/g;
    }
    return( $self->report_hw( "status $item ".($main::status_texts{$status})." ($date) ".localtime($date)." $comment" ) );
}


sub report_hw
{
    my ($self, $msg ) = @_;
    my ($timeout, $saddr );
    my( $res );

    $timeout = $self->{"timeout"};
    return(0) unless( $self->{"connected"} );

    $res = 0;
    eval {
	$SIG{'ALRM'} = sub { die };
	Platform::bsalarm($timeout); 
	    
	$msg .= "\n";
	print "reporter reporting: ".$msg if( $main::dl>5 );
	unless( send( $self->{"fh"}, $msg, 0 ) ) {
	    $res = 0;
	    $self->_end_report;
	}
	Platform::bsalarm(0);
	$res = 1;
    };
    Platform::bsalarm(0);
    return $res;
}


sub socket {
    my( $self ) = @_;

    return( $self->{"fh"} );
}

1;
